#
#

import ns3
import ospfwalk
import networkx
import pygraphviz
import iplib
#import visualizer

def PingRtt (context, rtt):
    print "Received Response with RTT = " + rtt;
    ns3.Simulator.Destroy ()


def main(argv):
    rlsas = []
    rlsas_dic = {}
    nlsas = []
    nlsas_dic = {}
    routers = []
    nets = []
    csma_dic = {}
    p2p_dic = {}
    num_links = 0
    nodes = ns3.NodeContainer();

    # Parse LSDB
#    ospfwalk.parse_show_ip_ospf_database_router(open("./show_ospf_db_r.txt", "r").read(), rlsas, rlsas_dic)
#    ospfwalk.parse_show_ip_ospf_database_network(open("./show_ospf_db_n.txt", "r").read(), nlsas, nlsas_dic)
    ospfwalk.parse_show_ip_ospf_database_router(open("./show_ospf_db_r_wide.txt", "r").read(), rlsas, rlsas_dic)
    ospfwalk.parse_show_ip_ospf_database_network(open("./show_ospf_db_n_wide.txt", "r").read(), nlsas, nlsas_dic)

    G = networkx.Graph()
    A = pygraphviz.AGraph(directed=False, strict=True)
    ospfwalk.conv_lsas_to_graph(rlsas, rlsas_dic, nlsas, nlsas_dic, {}, {}, G, A)

    # prepare for transit link
    csma = ns3.CsmaHelper();
    csma.SetChannelAttribute("DataRate", ns3.DataRateValue(ns3.DataRate("5Mbps")));
    csma.SetChannelAttribute("Delay", ns3.TimeValue(ns3.MilliSeconds(2)));

    p2p = ns3.PointToPointHelper();
    p2p.SetDeviceAttribute("DataRate", ns3.StringValue ("5Mbps"));
    p2p.SetChannelAttribute("Delay", ns3.TimeValue(ns3.MilliSeconds(2)));

    internet = ns3.InternetStackHelper();

    for node in G.nodes_iter():
#        print "node " + node
        if node.find('/') != -1:
            nets.append(node)
        else:
            routers.append(node)

    for router in routers:
        n = ns3.Node();
        internet.Install(n);
        nodes.Add(n);

#        print "router " + router
        try:
            rlsas = rlsas_dic[router]
        except KeyError:
            print "Warning: LSAs do not contain interface information about " + router + ". Ignoring."

        for rlsal in rlsas.links:
            if rlsal.type == "transit" :
                intaddr = rlsal.linkdata
                cost = rlsal.metric
#                print "IF: " + str(intaddr) + " cost:" + str(cost)

                try:
                    nlsa = nlsas_dic[rlsal.linkid]
                except KeyError:
                    print "Couldn't find Network LSA"
                    continue;

                net = iplib.CIDR(rlsal.linkdata + nlsa.netmask)
                netaddr = str(net.get_network_ip())
                netmask = str(net.get_netmask())

                try:
                    chan = csma_dic[netaddr+"/"+netmask]
                    dev = csma.Install(n,chan)
#                    csma.EnablePcap("lsdb-read", dev, False)
#                    print "Reuse existing channel " + netaddr+"/"+netmask + " IF: " + intaddr
                except KeyError:
#                    print "Added new chan"
                    dev = csma.Install(n)
#                    csma.EnablePcap("lsdb-read", dev, False)
                    csma_dic[netaddr+"/"+netmask] = dev.Get(0).GetChannel()

                ipv4 = n.GetObject(ns3.Ipv4.GetTypeId())
                idx = ipv4.AddInterface(dev.Get(0))
#                print "add if " + str(idx) + " node " + str(n.GetId()) + " if " + intaddr
                ipv4.AddAddress(idx,
                                ns3.Ipv4InterfaceAddress(ns3.Ipv4Address(intaddr), ns3.Ipv4Mask(netmask)))
                ipv4.SetUp (idx)
                num_links += 1
            elif rlsal.type == "stub":
#                print "Stub, skipped"
                continue
            # PtP
            else:
                try:
                    pnode = p2p_dic[rlsal.linkid]
                    dev = p2p.Install(n,pnode)

                    prlsas = rlsas_dic[rlsal.linkid]
                    for prlsal in prlsas.links:
                        if prlsal.type != "ptp" :
                            continue
                        if prlsal.linkid == router:
                            pintaddr = prlsal.linkdata
                            break

                    # edge[0]
                    ipv4 = n.GetObject(ns3.Ipv4.GetTypeId())
                    idx = ipv4.AddInterface(dev.Get(0))
#                    print "add if p0 " + str(idx) + " node " + str(n.GetId()) + " if " + rlsal.linkdata
                    ipv4.AddAddress(idx,
                                    ns3.Ipv4InterfaceAddress(ns3.Ipv4Address(rlsal.linkdata), ns3.Ipv4Mask("255.255.255.255")))
                    ipv4.SetUp (idx)

                    # edge[1]
                    ipv4 = pnode.GetObject(ns3.Ipv4.GetTypeId())
                    idx = ipv4.AddInterface(dev.Get(1))
#                    print "add if p1 " + str(idx) + " node " + str(pnode.GetId()) + " if " + pintaddr
#                    print "Install = " + str(dev.Get(1).GetIfIndex()) + " ifidx1 " + str(idx)
                    ipv4.AddAddress(idx,
                                    ns3.Ipv4InterfaceAddress(ns3.Ipv4Address(pintaddr), ns3.Ipv4Mask("255.255.255.255")))
                    ipv4.SetUp (idx)
                    num_links += 1
                except KeyError:
#                    print "INFO: Stores for p2p link " + router + ". "
                    p2p_dic[router] = n


    cmd = ns3.CommandLine();
    cmd.useViz = True
    cmd.AddValue("useViz", "Use visualizer (default=false")
    cmd.Parse(argv);


    print "#Nodes " + str(nodes.GetN()) + " #Links " + str(num_links) + " #Chans " + str(len(csma_dic))

    # Create OSPF
    processManager = ns3.ProcessManagerHelper()
    processManager.Install (nodes)
    quagga = ns3.QuaggaHelper();
    quagga.EnableOspf (nodes);
    quagga.Install (nodes);

    # Ping from Node 0 to Node N
    v4ping = ns3.V4Ping ()
    v4ping.SetAttribute ("Verbose", ns3.BooleanValue (True))
    # To idx=1 of Node(N)
    ipv4 = nodes.Get (0).GetObject(ns3.Ipv4.GetTypeId())
    print "Src: " + str(ipv4.GetAddress (1, 0).GetLocal ())
    ipv4 = nodes.Get (25).GetObject(ns3.Ipv4.GetTypeId())
    ipv4 = nodes.Get (nodes.GetN () - 1).GetObject(ns3.Ipv4.GetTypeId())
    print "Dst: " + str(ipv4.GetAddress (1, 0).GetLocal ())
    v4ping.SetAttribute ("Remote", ns3.Ipv4AddressValue (ipv4.GetAddress (1, 0).GetLocal ()))
    nodes.Get (0).AddApplication (v4ping)
    v4ping.SetStartTime (ns3.Seconds (1.0))

    ns3.Names.Add ("v4ping", v4ping);
#    ns3.Config.Connect ("/Names/v4ping/Rtt", generate_callback_classes (PingRtt));

#    ns3.Ipv4GlobalRoutingHelper.PopulateRoutingTables ()

    # Run Simulation     
#    visualizer.start ()
    ns3.Simulator.Stop(ns3.Seconds(1800.0))
    ns3.Simulator.Run()

    ns3.Simulator.Destroy()

if __name__ == '__main__':
    import sys
    main(sys.argv)

